/******************************************************************************
Copyright (C) 2002, 2003 Red Hat, Inc.

This file is part of op_visualise, a part of Oprofile.

This file may be redistributed and/or modifed under the terms of the GNU
General Public License as published by the Free Software Foundation; either
version 2, or (at your option) any later version.

op_visualise is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

op_visualise was written, and is maintained, by Chris Moller,
Red Hat Inc, cmoller@redhat.com

This file provides system-wide oprofile operations.

******************************************************************************/

/* #define USE_FUNCTION_TREE_SHOW */

#define __USE_GNU	/* enables GNU C extensions */
#define _GNU_SOURCE	/* enables GNU C extensions */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include "op_visualise.h"

#ifndef DONT_USE_OBSTACKS
static struct obstack composite_obstack;
static int            composite_obstack_initted = 0;
#endif

static int max_ctr_val =  -1;

#ifdef USE_FUNCTION_TREE_SHOW
static gboolean
composite_tree_show(gpointer key, gpointer value, gpointer data __attribute__ ((unused)))
{
  int i;
  fprintf(stderr,"tree key = \"%s\"\n", (char *)key);
  fprintf(stderr,"tree value = \"%s\" [%u]\n", 
		  ((composite_s *)value)->sample_fn,
		  ((composite_s *)value)->total_values);
  for (i = 0; i < ((composite_s *)value)->values_next; i++);
    fprintf(stderr,"\t%d: %u\n", i, ((composite_s *)value)->values[i]);
  return FALSE;
}
#endif

void
build_composite_tree()		/* fixme -- add call to this from sample-filessel.c */
{
  struct dirent * dir_ent;
  DIR * samples_DIR;
  char * rfn;
  char * eon;
  int ctr_val;
  composite_s * hit;
  int rc;
  char * samples_dir = files_in_use[SAMPLES_DIR] ? : DEFAULT_SAMPLES_DIR;

  if (!(samples_DIR = opendir(samples_dir))) {
    gtk_label_set(GTK_LABEL(status_bar), "Missing samples directory.");
    return;
  }
 
  if (composite_tree) g_tree_destroy(composite_tree);
  composite_tree = g_tree_new(function_tree_compare);
#ifndef DONT_USE_OBSTACKS
  if (composite_obstack_initted) obstack_free(&composite_obstack, NULL);
  obstack_init(&composite_obstack);
  composite_obstack_initted = 1;
#endif

  max_v = 0;

  while ((dir_ent = readdir(samples_DIR))) {
    if ((DT_REG == dir_ent->d_type) && ('}' == *dir_ent->d_name)) {
      rfn = obstack_copy0(&composite_obstack, dir_ent->d_name, strlen(dir_ent->d_name));
      if ((eon = strchr(rfn, '#'))) {
	*eon = '\0';
	ctr_val = atoi(eon+1);
      }
      else {
	ctr_val = 0;
	continue;
      }
      if (ctr_val >= max_ctr_val) max_ctr_val = ctr_val;
      if ((hit = g_tree_lookup(composite_tree, rfn))) {
        if (ctr_val >= hit->values_max) {
	  odb_value_t * kvals = hit->values;
	  int          tvals = hit->values_next;
	  hit->values_max += COMPOSITE_VALUES_INCR;
	  hit->values = obstack_alloc(&composite_obstack, hit->values_max * sizeof(composite_s));
	  hit->total_values = 0;
	  if (kvals) memcpy(hit->values, kvals, tvals * sizeof(composite_s));
	}
      }
      else {
	hit = obstack_alloc(&composite_obstack, sizeof(composite_s));
	hit->sample_fn = rfn;
	hit->values_max = ctr_val + 1;
	hit->values_next = ctr_val;
	hit->values = obstack_alloc(&composite_obstack, hit->values_max * sizeof(composite_s));
	bzero(hit->values, hit->values_max * sizeof(composite_s));
      }
      g_tree_insert(composite_tree, (gpointer)rfn, hit);
      
      rc = composite_sample_file(dir_ent->d_name, hit, ctr_val);
      if (EXIT_SUCCESS != rc) {
        gtk_label_set(GTK_LABEL(status_bar), "Error examining sample file.");
      }
      else {
        if (hit->total_values > max_v) max_v = hit->total_values;
      }
    }
  }
  closedir(samples_DIR);
#ifdef USE_FUNCTION_TREE_SHOW
  g_tree_foreach(composite_tree, composite_tree_show, NULL);
#endif
}
