/******************************************************************************
Copyright (C) 2002, 2003 Red Hat, Inc.

This file is part of op_visualise, a part of Oprofile.

This file may be redistributed and/or modifed under the terms of the GNU
General Public License as published by the Free Software Foundation; either
version 2, or (at your option) any later version.

op_visualise is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details. 

op_visualise was written, and is maintained, by Chris Moller,
Red Hat Inc, cmoller@redhat.com

This file provides functions that interface to the oprofile db handler.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "op_visualise.h"

#ifndef DONT_USE_OBSTACKS
static struct obstack samples_obstack;
#endif
static int samples_obstack_initted = 0;
static int op_samples_count;

static void
show_callback(odb_key_t key,
	      odb_value_t value,
	      void * data __attribute__ ((unused)))
{
  bfd_vma      t_key = (unsigned int)(start_vma + key);
  unsigned int t_val = (unsigned int)value;
  op_samples_s op_samples_t;
  
  if (min_key > t_key)   min_key   = t_key;
  if (max_key < t_key)   max_key   = t_key;
  if (min_value > t_val) min_value = t_val;
  if (max_value < t_val) max_value = t_val;
  total_hits += t_val;
  
  op_samples_t.value = t_val;
  op_samples_t.loc = t_key;
#ifndef DONT_USE_OBSTACKS
  obstack_grow(&samples_obstack, &op_samples_t, sizeof(op_samples_s));
#endif  
}

int
copy_sample_file(char * samples_file)
{
  samples_odb_t stree;
  char * err_msg;
  int rc;
  
  min_key = 0xffffffff;
  max_key = 0;
  min_value = 0xffffffff;
  max_value = 0;
  total_hits = 0;

  rc = odb_open(&stree, samples_file, ODB_RDONLY, sizeof(struct opd_header), &err_msg);
  if (EXIT_SUCCESS == rc) {
#ifndef DONT_USE_OBSTACKS
    if (samples_obstack_initted) obstack_free(&samples_obstack, NULL);
    obstack_init(&samples_obstack);
    samples_obstack_initted = 1;
#endif  

    samples_odb_travel(&stree, 0, ~0, show_callback, 0);
    odb_close(&stree);

#ifndef DONT_USE_OBSTACKS
    op_samples_count = obstack_object_size(&samples_obstack)/sizeof(op_samples_s);
    op_samples = obstack_finish(&samples_obstack);
#endif  
  }

  return rc;	/* fixme -- prolly needs work */
}

static int
op_samples_search_fcn(const void * a, const void * b)
{
  bfd_vma aa = (bfd_vma)a;
  bfd_vma bb = ((op_samples_s *)b)->loc;
  
  return (aa > bb) - (aa < bb);
}

/* swiped from GNU libc and modified by chlm */

static void *
bsearch_modified (const void *key, const void *base, size_t nmemb, size_t size,
		  int (*compar) (const void *, const void *),
		  size_t * r_idx_p, size_t * r_l_p, size_t * r_u_p)
{
  size_t l, u, idx;
  const void *p;
  int comparison;

  l = 0;
  u = nmemb;
  while (l < u)
    {
      idx = (l + u) / 2;
      if (r_idx_p) *r_idx_p = idx;
      if (r_l_p)   *r_l_p = l;
      if (r_u_p)   *r_u_p = u;
      p = (void *) (((const char *) base) + (idx * size));
      comparison = (*compar) (key, p);
      if (comparison < 0)
        u = idx;
      else if (comparison > 0)
        l = idx + 1;
      else
        return (void *) p;
    }

  return NULL;
}


int
samples_locate_nearest_in_bound(bfd_vma target,
				unsigned int window)
{
  op_samples_s * ss;
  size_t r_idx, r_l, r_u;
  int t_least;
  unsigned int least;
  int least_idx = -1;
  int rc;

  rc = -1;
  if ((ss = bsearch_modified((const void *)target, op_samples,
			     op_samples_count, sizeof(op_samples_s),
			     op_samples_search_fcn, &r_idx, &r_l, &r_u))) {
    rc = (int)r_idx;
  }
  else {
    int i;
    least = 0x7fffffff;
    for (i = (int)r_l - 2; i <= (int)r_u + 2; i++) {
      if ((i >= 0) && (i < op_samples_count)) {
	if (least > (t_least = abs(op_samples[i].loc - target))) {
	  least = t_least;
	  least_idx = i;
	}
      }
    }
    if (least <= window) rc = least_idx;
  }

  return rc;
}

void
samples_travel(unsigned int min_l,
	       unsigned int max_l,
	       void (*callback)(),
	       GdkGC * gc)
{
  int i;
  
  int min_idx = samples_locate_nearest_in_bound(min_l, 0xffffffff);
  int max_idx = samples_locate_nearest_in_bound(max_l, 0xffffffff);

  for (i = min_idx; i <= max_idx; i++) (*callback)(op_samples[i].loc, op_samples[i].value, gc);
}

EXTERN GTree * composite_tree INITIAL(NULL);


static odb_value_t tv;

static void
composite_callback(odb_key_t key __attribute__ ((unused)),
	           odb_value_t value __attribute__ ((unused)),
	           void * data __attribute__ ((unused)))
{
  tv += value;
  *((odb_value_t *)data) += value;
}

int
composite_sample_file(char * fn, composite_s * hit, int ctr_val)
{
  samples_odb_t stree;
  char * qfn;
  char * err_msg;
  int rc;

  hit->values[ctr_val] = 0;
  tv = 0;
  asprintf(&qfn,"%s/%s", files_in_use[SAMPLES_DIR], fn);
  rc = odb_open(&stree, qfn, ODB_RDONLY, sizeof(struct opd_header), &err_msg);
  if (EXIT_SUCCESS == rc) {
    samples_odb_travel(&stree, 0, ~0, composite_callback, &(hit->values[ctr_val]));
    odb_close(&stree);
    hit->total_values += tv;
  }
  free(qfn);
  return rc;
}

