/******************************************************************************
Copyright (C) 2002, 2003 Red Hat, Inc.

This file is part of op_visualise, a part of Oprofile.

This file may be redistributed and/or modifed under the terms of the GNU
General Public License as published by the Free Software Foundation; either
version 2, or (at your option) any later version.

op_visualise is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

op_visualise was written, and is maintained, by Chris Moller,
Red Hat Inc, cmoller@redhat.com

This file provides both standard and custom file selection functions for 
op_visualise.

******************************************************************************/

#define __USE_GNU	/* enables GNU C extensions */
#define _GNU_SOURCE	/* enables GNU C extensions */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include "op_visualise.h"

extern GtkWidget * status_bar;

static GtkWidget * file_sel;			/* non-sample filesel window		*/


/*									*/
/* accept/open the new file						*/
/*									*/


static void
file_select_ok (GtkObject * object __attribute__ ((unused)),
		gpointer    pdata)
{
  struct stat buf;
  gint data = (long int)pdata;
  
  GtkFileSelection * fs = GTK_FILE_SELECTION(file_sel);
  const char * fn = gtk_file_selection_get_filename(fs);

  if (0 != stat(fn, &buf)) {
    gtk_label_set(GTK_LABEL(status_bar), "Error stating file.");
    return;
  }

  switch (data) {
  case BINARY_FILE:
    if (!S_ISREG(buf.st_mode)) {
      gtk_label_set(GTK_LABEL(status_bar), "Non-regular file selected.");
      return;
    }

    if (!(S_IROTH & buf.st_mode)) {
      gtk_label_set(GTK_LABEL(status_bar), "Selected file not readable.");
      return;
    }
    break;
  case SAMPLES_DIR:
  case BINARIES_DIR:
  case SOURCES_DIR:
    if (!S_ISDIR(buf.st_mode)) {
      gtk_label_set(GTK_LABEL(status_bar), "Non-directory selected.");
      return;
    }

    if (!(S_IROTH & buf.st_mode)) {
      gtk_label_set(GTK_LABEL(status_bar), "Selected directory not scanable.");
      return;
    }
    break;
  }

  if (!files_in_use[data] || strcmp(fn, files_in_use[data])) {
    char * ep;
    if (files_in_use[data]) free(files_in_use[data]);
    files_in_use[data] = strdup(fn);
    ep = files_in_use[data] + (strlen(files_in_use[data]) - 1);
    if (('/' == *ep) && ('\0' != *(ep+1)))*ep = '\0';
    file_changed |= (1 << data);
    switch (data) {
    case BINARY_FILE:
      gtk_label_set(GTK_LABEL(info_labels[data]), fn);
      break;
    case SAMPLES_DIR:
      build_composite_tree();
      break;
#ifdef USE_SOURCES_BUTTON
    case SOURCES_DIR:
      gtk_label_set(GTK_LABEL(info_labels[data]), fn);
      break;
#endif
    }
  }
		       
  gtk_label_set(GTK_LABEL(status_bar), "");
  gtk_widget_destroy(GTK_WIDGET(file_sel));
  window_open &= ~(1 << data);

  if (file_changed) {
    switch (data) {
    case BINARY_FILE:
      maybe_visualise_data();
      break;
    }
  }
}


/*									*/
/* cancel file selection						*/
/*									*/

static void
file_select_cancel (GtkObject * object __attribute__ ((unused)),
		    gpointer    pdata)
{
  gtk_widget_destroy(file_sel);
  window_open &= ~(1 << (long int)pdata);
}


/*									*/
/*									*/
/*									*/
/* normal select main							*/
/*									*/
/*									*/
/*									*/

void
handle_select_file(GtkWidget * w __attribute__ ((unused)),
		   gint gdata,
		   void * callback_action __attribute__ ((unused)))
{
  long int data = (long int)gdata;
  char * wname = 0;
  char * dname = 0;

  if (!(window_open & (1 << data))) {
    window_open |= (1 << data);
    
    switch (data) {
    case BINARIES_DIR:
      wname = "Select Binary File";
      dname = files_in_use[BINARIES_DIR];
      break;
    case BINARY_FILE:
      wname = "Select Binary File";
      dname = files_in_use[BINARIES_DIR];
      break;
    case SAMPLES_DIR:
      dname = files_in_use[SAMPLES_DIR];
      wname = "Select Samples Directory";
      break;
    case SOURCES_DIR:
      dname = files_in_use[SOURCES_DIR];
      wname = "Select Sources Directory";
      break;
    }

    file_sel = gtk_file_selection_new(wname);
    if (dname) chdir(dname);
    gtk_signal_connect(GTK_OBJECT(file_sel),
		     "destroy",
		     GTK_SIGNAL_FUNC(file_select_cancel),
		     (gpointer)data);
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_sel));
    gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->ok_button), "clicked",
			GTK_SIGNAL_FUNC (file_select_ok),
			(gpointer)data);
    gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->cancel_button), "clicked",
			GTK_SIGNAL_FUNC (file_select_cancel),
			(gpointer)data);
    gtk_widget_show(file_sel);
  }
}

