/******************************************************************************
Copyright (C) 2002, 2003 Red Hat, Inc.

This file is part of op_visualise, a part of Oprofile.

This file may be redistributed and/or modifed under the terms of the GNU
General Public License as published by the Free Software Foundation; either
version 2, or (at your option) any later version.

op_visualise is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details. 

op_visualise was written, and is maintained, by Chris Moller,
Red Hat Inc, cmoller@redhat.com

This file provides common definitions and declarations used by op_visualise.

******************************************************************************/

#ifndef OP_VISUALISE_H
#define OP_VISUALISE_H

#ifdef DO_INITIALISE
#define EXTERN
#define INITIAL(v) = (v)
#else
#define EXTERN extern
#define INITIAL(v)
#endif

#include <libiberty.h>
#include <gtk/gtk.h>
#include <odb_hash.h>
#include <op_sample_file.h>

/* make sure they don't end in a '/' (except root) */
#define DEFAULT_SAMPLES_DIR	"/var/lib/oprofile/samples"
#define DEFAULT_BINARIES_DIR	"/"
#define DEFAULT_INITS_FILE	"./.opinit"

#define MAIN_WINDOW_DEFAULT_WIDTH  600
#define MAIN_WINDOW_DEFAULT_HEIGHT 600

#define DRAWING_AREA_DEFAULT_WIDTH  550
#define DRAWING_AREA_DEFAULT_HEIGHT 400

EXTERN gint main_window_width  INITIAL(MAIN_WINDOW_DEFAULT_WIDTH);
EXTERN gint main_window_height INITIAL(MAIN_WINDOW_DEFAULT_HEIGHT);

EXTERN gint drawing_area_width  INITIAL(DRAWING_AREA_DEFAULT_WIDTH);
EXTERN gint drawing_area_height INITIAL(DRAWING_AREA_DEFAULT_HEIGHT);

/* s.b. <...> for system incl; using lcl now as workaround */
#include "bfd.h"

#ifndef DONT_USE_OBSTACKS
#include <obstack.h>
#define obstack_chunk_alloc xmalloc
#define obstack_chunk_free free
#endif

#ifdef DO_MEM_CHECK
#include <mcheck.h>
#endif

typedef struct _sample_s {
  char * sample_file;
  char * le[3];
} sample_s;

#define SAMPLE_EXEC_NAME	0
#define SAMPLE_CTR		1
#define SAMPLE_EXEC_PATH	2

#define SAMPLE_INCR 8

EXTERN sample_s ** sample_list INITIAL(NULL);
EXTERN int sample_next INITIAL(0);
EXTERN int sample_max INITIAL(0);		/* fixme -- not needed in obstack version */

EXTERN asection ** text_section_list INITIAL(NULL);
EXTERN int text_section_next INITIAL(0);
EXTERN asymbol **syms;
EXTERN bfd * ibfd INITIAL(NULL);

EXTERN int stree_valid INITIAL(0);

EXTERN char * files_in_use[]
#ifdef DO_INITIALISE
= {0, 0, 0, 0, 0}
#endif
;

#define SAMPLES_FILE	0
#define BINARY_FILE	1
#define SOURCES_DIR	2
#define SAMPLES_DIR	3
#define BINARIES_DIR	4
#define PRINT_FILE	5

EXTERN int window_open  INITIAL(0);
EXTERN int file_changed INITIAL(0);

typedef struct {
  char * sample_fn;
  odb_value_t total_values;
  odb_value_t * values;
  int values_next;
  int values_max;
} composite_s;

EXTERN GTree * composite_tree INITIAL(NULL);
#define COMPOSITE_VALUES_INCR	4
EXTERN odb_value_t max_v;

int      get_samples();
void     handle_open_sample(GtkWidget * w, gint data, void * callback_action);
void     handle_select_file(GtkWidget * w, gint data, void * callback_action);
#ifdef ENABLE_PLOT
void     handle_print_file(GtkWidget * w, gint data, void * callback_action);
#endif
void     maybe_visualise_data(void);
int      get_samples(void);
void     draw_chart(void);
void     initialise_progress_bar(double);
gboolean update_progress_bar(double);
void     remove_progress_bar(void);
int      copy_sample_file(char *);
void     samples_travel(unsigned int, unsigned int, void (*callback)(), GdkGC *);
int      samples_locate_nearest_in_bound(bfd_vma, unsigned int);
void     display_callback(bfd_vma, unsigned int, GdkGC *);
int      function_tree_compare(gconstpointer, gconstpointer);
void     build_composite_tree(void);
int      composite_sample_file(char *, composite_s *, int);
void	 update_labels(void);

EXTERN GtkWidget * info_labels[3];
EXTERN GtkWidget * status_bar;

EXTERN GtkWidget * drawing_area;
EXTERN GtkWidget * main_window;
EXTERN GdkPixmap * pixmap INITIAL(NULL);
EXTERN GdkFont * fixed_font;

EXTERN GtkWidget * vrule;
EXTERN GtkWidget * left_label;
EXTERN GtkWidget * right_label;
EXTERN char * left_label_text  INITIAL(NULL);
EXTERN char * right_label_text INITIAL(NULL);

EXTERN GtkObject * adjustment;

EXTERN GdkFont * font;
EXTERN GdkGC * red_gc;
EXTERN GdkGC * green_gc;
EXTERN GdkGC * yellow_gc;


typedef struct {
  bfd_vma loc;
  unsigned int value;
} op_samples_s;

EXTERN op_samples_s * op_samples INITIAL(NULL);

EXTERN unsigned int min_key, max_key;
EXTERN bfd_vma min_value, max_value;
EXTERN double max_value_d;
EXTERN unsigned int max_value_l;
EXTERN unsigned int total_hits;
EXTERN double min_key_d,   max_key_d;
EXTERN double min_ruler_d, max_ruler_d;
EXTERN double x_scale, y_scale;

EXTERN bfd_vma start_vma;

EXTERN int sym_count;
EXTERN symbol_info ** symbol_list INITIAL(NULL);

#define PROGRESS_BAR_THRESHOLD 500
#define PROGRESS_BAR_TICK      0.02
EXTERN gboolean long_op_in_progress INITIAL(FALSE);


typedef struct {
  char * function_name;
  bfd_vma min_loc;
  bfd_vma max_loc;
  unsigned int total_hits;
} function_data_s;

EXTERN GTree * function_tree INITIAL(NULL);
EXTERN GTree * file_tree INITIAL(NULL);

EXTERN gboolean highlight_active INITIAL(FALSE);
EXTERN bfd_vma  highlight_min;
EXTERN bfd_vma  highlight_max;

#endif /* OP_VISUALISE_H */

